@@ -2,21 +2,19 @@
 mod rustls;
 
 #[cfg(feature = "tlsrust")]
-use self::rustls::{UnderlyingTLSStream, UnderlyingAccept, TLSConfig};
+pub use self::rustls::{ParsedTLSConfig, TlsUserConfig};
 #[cfg(feature = "tlsrust")]
-pub use self::rustls::{TlsUserConfig, ParsedTLSConfig};
+use self::rustls::{TLSConfig, UnderlyingAccept, UnderlyingTLSStream};
 
 use super::{PlainIncoming, PlainStream};
 use core::task::{Context, Poll};
-use std::pin::Pin;
-use hyper::server::accept::Accept;
 use futures_util::ready;
-#[cfg(feature = "tlsrust_acme")]
-use tokio_rustls::rustls::Session;
+use hyper::server::accept::Accept;
+use std::future::Future;
 use std::io;
+use std::pin::Pin;
 use std::sync::Arc;
 use tokio::io::{AsyncRead, AsyncWrite, ReadBuf};
-use std::future::Future;
 
 use std::net::SocketAddr;
 
@@ -33,10 +31,22 @@ enum State {
 
 pub(crate) trait TLSBuilderTrait {
     /// called by first vHost that wants TLS
-    fn new(c: &TlsUserConfig, sni: Option<&str>) -> Result<Self, Box<dyn std::error::Error + Send + Sync>> where Self: std::marker::Sized;
+    fn new(
+        c: &TlsUserConfig,
+        sni: Option<&str>,
+    ) -> Result<Self, Box<dyn std::error::Error + Send + Sync>>
+    where
+        Self: std::marker::Sized;
     /// called by all but the first vHost on one socket
-    fn add(&mut self, c: &TlsUserConfig, sni: Option<&str>) -> Result<(), Box<dyn std::error::Error + Send + Sync>>;
-    fn get_accept_feature(accept: &TlsAcceptor, stream: PlainStream) -> UnderlyingAccept<PlainStream>;
+    fn add(
+        &mut self,
+        c: &TlsUserConfig,
+        sni: Option<&str>,
+    ) -> Result<(), Box<dyn std::error::Error + Send + Sync>>;
+    fn get_accept_feature(
+        accept: &TlsAcceptor,
+        stream: PlainStream,
+    ) -> UnderlyingAccept<PlainStream>;
     fn get_acceptor(self, incoming: PlainIncoming) -> TlsAcceptor;
 }
 
@@ -74,7 +84,7 @@ impl AsyncRead for TlsStream {
             State::Handshaking(ref mut accept) => match ready!(Pin::new(accept).poll(cx)) {
                 Ok(mut stream) => {
                     #[cfg(feature = "tlsrust_acme")]
-                    if stream.get_ref().1.get_alpn_protocol() == Some(ACME_TLS_ALPN_NAME) {
+                    if stream.get_ref().1.alpn_protocol() == Some(ACME_TLS_ALPN_NAME) {
                         log::debug!("completed acme-tls/1 handshake");
                         return Pin::new(&mut stream).poll_shutdown(cx);
                         //EOF
@@ -156,4 +166,3 @@ impl Accept for TlsAcceptor {
         }
     }
 }
-
